#!/bin/sh -x

rootDir=`pwd`
boost_root=/usr/local/boost_1_53_0
boost_stage=/usr/local/lib
boost_lib_ext=dylib

# Boost my local
BOOST_INCLUDEDIR=/usr/local/include
BOOST_LIBRARYDIR=/usr/local/lib
boost_stage=/usr/local/lib
Boost_LIBRARIES="/usr/local/lib/libboost_filesystem.so.1.53.0;/usr/local/lib/libboost_system.so.1.53.0;/usr/local/lib/libboost_thread.so.1.53.0"
# End of Boost on my local

boost_gxx=xgcc40
zlib_root=${rootDir}/zlib
tinyXML_root=${rootDir}/tinyxmldll
HyperNEAT_root=${rootDir}/HyperNEAT
JGTL_include=${rootDir}/JGTL/include
ode_root=/usr/local

# Config on Moran
if [[ `hostname` = mmmlog* ]]
then
    echo On Mt. Moran
    module load CMAKE/cmake-2.8.9
#    module load BOOST/boost_1_51_0_gcc
    module load BOOST/boost_1_51_0_intel


    module load INTEL/2012.0.032/compilers

    CC=icc
    CXX=icpc

    export CC=icc
    export CXX=icpc

    echo Setting boost path variables
#    BOOST_INCLUDEDIR=/apps/BOOST/boost_1_51_0_gcc/include
#    BOOST_LIBRARYDIR=/apps/BOOST/boost_1_51_0_gcc/lib
#    boost_stage=/apps/BOOST/boost_1_51_0_gcc/lib
#    Boost_LIBRARIES="/apps/BOOST/boost_1_51_0_gcc/lib/libboost_filesystem.so.1.51.0;/apps/BOOST/boost_1_51_0_gcc/lib/libboost_system.so.1.51.0;/apps/BOOST/boost_1_51_0_gcc/lib/libboost_thread.so.1.51.0"

    BOOST_INCLUDEDIR=/apps/BOOST/boost_1_51_0_intel-linux/include
    BOOST_LIBRARYDIR=/apps/BOOST/boost_1_51_0_intel-linux/lib
    boost_stage=/apps/BOOST/boost_1_51_0_intel-linux/lib/
    Boost_LIBRARIES="/apps/BOOST/boost_1_51_0_intel-linux/lib/libboost_filesystem.so.1.51.0;/apps/BOOST/boost_1_51_0_intel-linux/lib/libboost_system.so.1.51.0;/apps/BOOST/boost_1_51_0_intel-linux/lib/libboost_thread.so.1.51.0"
fi

echo Building required packages
echo

# Build TinyXML
if test -e ${tinyXML_root}/out/libtinyxmldll.${boost_lib_ext}
then
    echo TinyXML exists.  Skipping
    echo
else
    echo Building zlib
    cd ${zlib_root}
    cmake .
    make
    echo Finished making zlib

    echo Building TinyXML
    cd ${tinyXML_root}
    cmake .
    make
    echo Finished making TinyXML
    echo
fi

# Test ODE  
if test -e ${ode_root}/lib/libode.a
then
    echo ODE found, using ODE.
    echo
    compile_ode="true"
else
    echo ODE not found. Ignore ODE.
    echo
    compile_ode="false"
fi


# Build HyperNEAT
if test -e ${HyperNEAT_root}/out/Hypercube_NEAT
then
    echo Hypercube_NEAT exists.  Skipping
    echo
else
    echo Building HyperNEAT
    cd ${HyperNEAT_root}
    cmake -DBoost_LIBRARY_DIRS:string=${BOOST_LIBRARYDIR} -DBoost_INCLUDE_DIRS:string=${BOOST_INCLUDEDIR} .
    if [ "$compile_ode" == "false" ]
    then
	cmake -DToODEorNot:string=   #purposefully empty so ode does not get linked in
    else
	cmake -DToODEorNot:string=ode
    fi

    if [ "$compile_ode" == "false" ]
    then
	echo "Ignoring ODE"
	cmake -DCMAKE_INSTALL_PREFIX=${rootDir}/out -DJGTL_INCLUDE=${JGTL_include} -DBoost_LIBRARY_DIRS:string=${BOOST_LIBRARYDIR} -DBoost_INCLUDE_DIRS:string=${BOOST_INCLUDEDIR} -DBOOST_ROOT=${boost_root} -DBOOST_STAGE:string=${boost_stage} -DBoost_LIBRARIES=${Boost_LIBRARIES} -DEXECUTABLE_OUTPUT_PATH=${rootDir}/out -DJGTL_INCLUDE=${JGTL_include} -DLIBRARY_OUTPUT_PATH:string=${rootDir}/hyperneatLibs -DTINYXMLDLL_INCLUDE=${tinyXML_root}/include -DTINYXMLDLL_LIB=${tinyXML_root}/out -DCMAKE_COMPILER_IS_GNUCXX:string=/opt/intel/cce/10.0.025/bin/icpc
    else
	cmake -DCOMPILE_ODE=ON -DCMAKE_INSTALL_PREFIX=${rootDir}/out -DJGTL_INCLUDE=${JGTL_include} -DBoost_LIBRARY_DIRS:string=${BOOST_LIBRARYDIR} -DBoost_INCLUDE_DIRS:string=${BOOST_INCLUDEDIR} -DBOOST_ROOT=${boost_root} -DBOOST_STAGE:string=${boost_stage} -DBoost_LIBRARIES=${Boost_LIBRARIES} -DEXECUTABLE_OUTPUT_PATH=${rootDir}/out -DJGTL_INCLUDE=${JGTL_include} -DLIBRARY_OUTPUT_PATH:string=${rootDir}/hyperneatLibs -DTINYXMLDLL_INCLUDE=${tinyXML_root}/include -DTINYXMLDLL_LIB=${tinyXML_root}/out -DDRAWSTUFF_INCLUDE:string=${rootDir}/ode-0.10.1/include -DODE_LIB:string=${ode_root}/lib -DODE_INCLUDE:string=${ode_root}/include -DCMAKE_COMPILER_IS_GNUCXX:string=/opt/intel/cce/10.0.025/bin/icpc
    fi
    make
    cd ${rootDir}

    echo Finished making Hypercube_NEAT
    echo
fi

# Run Target-Pattern Experiment
#cd ./out
#./Hypercube_NEAT -I ../ConfigFiles/TargetPattern-Experiment.dat -O TargetPattern-Experiment-Output.xml


