
#ifndef HCUBE_BARE_BONES_EXPERIMENT_H_INCLUDED
#define HCUBE_BARE_BONES_EXPERIMENT_H_INCLUDED

#include "HCUBE_Experiment.h"
#include <fstream>


namespace HCUBE
{
	
    class BareBonesExperiment : public Experiment
    {
    protected:
        NEAT::FastNetwork <double> substrate;
        int numNodesX,numNodesY;
		
        map<Node,string> nameLookup;
		
		int numSheets;
		
    public:

        BareBonesExperiment(string _experimentName);
		
        virtual ~BareBonesExperiment() {}
		
		/**
		 * Function that takes a population size and creates an initial population.
		 * 
		 * Overwrites createInitialPopulation in HCUBE_Experiment.
		 */
        virtual NEAT::GeneticPopulation* createInitialPopulation(int populationSize);
		
		double zeroOrNormalize(double mWeightToAlter);		
		
        virtual void generateSubstrate();
		
        /**
         * Function that takes a genetic individual and creates its 'substrate'.
         */
		NEAT::FastNetwork<double> populateAndReturnSubstrate(shared_ptr<const NEAT::GeneticIndividual> individual);
		
		
        double processEvaluation(shared_ptr<NEAT::GeneticIndividual> individual);
		
        virtual void processGroup(shared_ptr<NEAT::GeneticGeneration> generation);
		
        virtual void printSubstrate(shared_ptr<const NEAT::GeneticIndividual> individual, string fileName);
		
        virtual void printNetworkCPPN(shared_ptr<const NEAT::GeneticIndividual> individual, string fileName);
		
        virtual Experiment* clone();
		
    };
}

#endif // HCUBE_BARE_BONES_EXPERIMENT_H_INCLUDED
