#include "HCUBE_Defines.h"

#include "JGTL_CommandLineParser.h"

#include "HCUBE_ExperimentRun.h"

#include "Experiments/HCUBE_Experiment.h"
#include "Experiments/HCUBE_FindClusterExperiment.h"
//#include "Experiments/HCUBE_CheckersExperiment.h"
#ifdef ONAMAC
#include <ApplicationServices/ApplicationServices.h>
#endif //ONAMAC
#define PRINT_GENCHAMPFITNESSVIAPOSTHOCEVAL (0)

#define EPOCHS_PER_PRINT (100000)

void printHelp(){
	cout << "Syntax for passing command-line options to HyperNEAT (do not actually type '<' or '>' ):\n";
	cout << "./HyperNEAT -I <configfile> -O <outputfile>\n";
	cout << "Or, to restart a run:\n";
	cout << "./HyperNEAT -X <initialPopulationFile> -XG <numberOfExtraGenerations> -I <configfile>\n";
	cout << "Or, to run a post hoc analysis:\n";
	cout << "./HyperNEAT <populationFile>";
	cout << "Options: \n";
	cout << "-R <seed>            Sets the seed for this run.\n";
	cout << "-G <maxGeneration>   Sets the maximum number of generations for this run, overwriting the value in the config file.\n";
	cout << "-T <terminalFitness> If the average fitness is larger than this fitness, the experiment will terminate.\n";
}



using namespace HCUBE;
int HyperNEAT_main(int argc, char **argv) {

	CommandLineParser commandLineParser(argc, argv);

	//This is for memory debugging
	//_CrtSetBreakAlloc(137405);

	try {
		if (argc <= 1 || commandLineParser.HasSwitch("-h")) {
			printHelp();
		} else if (argc == 2) //running post-hoc eval
				{

			//Run the post-hoc analysis on every generation
			HCUBE::ExperimentRun experimentRun;

			experimentRun.setupExperimentInProgress(string(argv[1]), "");

			int numGenerations = experimentRun.getPopulation()->getGenerationCount();

			HCUBE::Experiment *experiment = experimentRun.getExperiment()->clone();
			if (false) //this block is for printing post hoc info on every generation, but not necessary for evaluating an org
			{
#ifndef PRINT_GENCHAMPFITNESSVIAPOSTHOCEVAL
				string outString = (erase_tail_copy(string(argv[1]),3)+string("_fitness.out"));
				cout << "Creating file " << outString << endl;
				ofstream outfile( outString.c_str() );
				string previousSummary;
				bool doOnce=false;
#endif 

				for (int generation = 0; generation < numGenerations; generation++) {
					//CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

					//CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

					if (generation
							&& (*(experimentRun.getIndividual(generation, 0).get()))
									== (*(experimentRun.getIndividual(generation - 1, 0).get()))) {
#ifndef PRINT_GENCHAMPFITNESSVIAPOSTHOCEVAL
						outfile << (generation+1) << ' ' << previousSummary << endl;
#endif
						continue;
					}

					shared_ptr < NEAT::GeneticIndividual > indiv = shared_ptr < NEAT::GeneticIndividual
							> (new NEAT::GeneticIndividual(*(experimentRun.getIndividual(generation, 0).get())));

					//CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

					//cout << "Beginning post-hoc evaluation " << (generation+1) << "/" << numGenerations << "...";
					//experiment->processIndividualPostHoc(indiv);
					//cout << "done!\n";

					//CREATE_PAUSE(string("Error!: ")+toString(__LINE__));
#ifndef PRINT_GENCHAMPFITNESSVIAPOSTHOCEVAL
					if (indiv->getUserData())
					{
						if (!doOnce)
						{
							doOnce=true;
							outfile
							<< "#Generation: "
							<< indiv->getUserData()->summaryHeaderToString()
							<< endl;
						}

						previousSummary = indiv->getUserData()->summaryToString();
						outfile << (generation+1) << ' ' << previousSummary << endl;
					}
					else
					{
						if (!doOnce)
						{
							doOnce=true;
							outfile
							<< "#Generation: "
							<< "Fitness:"
							<< endl;
						}

						previousSummary = toString(indiv->getFitness());
						outfile << (generation+1) << ' ' << previousSummary << endl;
					}
#endif

				}
			}
			int generation = numGenerations - 1; // jmc: interested in last generation (I don't know why it is -1)
			//                    int generation = 0; // jmc: interested in first generation
			cout << "Beginning post-hoc evaluation " << (generation) << "/" << numGenerations - 1 << "...\n";

			shared_ptr < NEAT::GeneticIndividual > indiv = shared_ptr < NEAT::GeneticIndividual
					> (new NEAT::GeneticIndividual(*(experimentRun.getIndividual(generation, 0).get())));

			experiment->processIndividualPostHoc(indiv);
			cout << "done!\n";

			/*
			 cout << "Done with run!  Running medium resolution tests...\n";
			 ((HCUBE::FindClusterExperiment*)experiment)->increaseResolution();

			 {
			 string outString = (erase_tail_copy(string(argv[1]),4)+string("_midres.out"));
			 ofstream outfile( outString.c_str() );
			 cout << "Creating file " << outString << endl;
			 string previousSummary;

			 bool doOnce=false;

			 for (int generation=0;generation<numGenerations;generation++)
			 {
			 //CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

			 //CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

			 if ( generation &&
			 (*(experimentRun.getIndividual(generation,0).get())) == (*(experimentRun.getIndividual(generation-1,0).get())) )
			 {
			 outfile << (generation+1) << ' ' << previousSummary << endl;
			 continue;
			 }

			 shared_ptr<NEAT::GeneticIndividual> indiv =
			 shared_ptr<NEAT::GeneticIndividual>(
			 new NEAT::GeneticIndividual(
			 *(experimentRun.getIndividual(generation,0).get())
			 )
			 );

			 //CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

			 cout << "Beginning post-hoc evaluation " << (generation+1) << "/" << numGenerations << "...";
			 experiment->processIndividualPostHoc(indiv);
			 cout << "done!\n";

			 //CREATE_PAUSE(string("Error!: ")+toString(__LINE__));

			 if (indiv->getUserData())
			 {
			 if (!doOnce)
			 {
			 doOnce=true;
			 outfile
			 << "#Generation: "
			 << indiv->getUserData()->summaryHeaderToString()
			 << endl;
			 }

			 previousSummary = indiv->getUserData()->summaryToString();
			 outfile << (generation+1) << ' ' << previousSummary << endl;
			 }

			 }
			 }
			 */
		} else if (commandLineParser.HasSwitch("-I") && commandLineParser.HasSwitch("-O")) {
			string inputFile = commandLineParser.GetSafeArgument("-I", 0, "input.dat");
			string popfile = commandLineParser.GetSafeArgument("-O", 0, "output.xml");

			//unsigned int randomSeed = stringTo<unsigned int>(commandLineParser.GetSafeArgument("-R",0,"0"));

			NEAT::Globals::init(inputFile);

			if (commandLineParser.HasSwitch("-R")) {
				NEAT::Globals::getSingleton()->seedRandom(
						stringTo<unsigned int>(commandLineParser.GetSafeArgument("-R", 0, "0")));
			}

			if (commandLineParser.HasSwitch("-G")) {
				SET_PARAMETER("MaxGenerations", atoi(commandLineParser.GetSafeArgument("-G", 0, "0").c_str()));
			}

			if (commandLineParser.HasSwitch("-T")) {
				SET_PARAMETER("TerminalFitness", atof(commandLineParser.GetSafeArgument("-T", 0, "0").c_str()));
			}

			int experimentType = int(GET_PARAMETER("ExperimentType") + 0.001);

			cout << "Loading Experiment: " << experimentType << endl;
			cout << "With Generations: " << GET_PARAMETER("MaxGenerations") << endl;

			//jmc: remove the file to make the way for the creation of a new one for this run
			ofstream output_file;
			output_file.open("gen-Genchamp-AvgFit.txt", ios::trunc);
			output_file.close();

			HCUBE::ExperimentRun experimentRun;

			experimentRun.setupExperiment(experimentType, popfile);

			cout << "Experiment set up\n";

			experimentRun.createPopulation();

			experimentRun.setCleanup(true);

			cout << "Population Created\n";

			experimentRun.start();
		} else if (commandLineParser.HasSwitch("-X")) {
			int extraGenerationsToRun;
			if (commandLineParser.HasSwitch("-XG")) {
//			NEAT::Globals::getSingleton()->seedRandom(stringTo<unsigned int>(commandLineParser.GetSafeArgument("-R",0,"0")));
				extraGenerationsToRun = atoi(commandLineParser.GetSafeArgument("-XG", 0, "0").c_str());
				cout << "This population will be evolved for the following number of additional generations: "
						<< extraGenerationsToRun << endl;
			} else {
				cout
						<< "You must specify how many more generations you want to run for when restarting a run from a saved population file."
						<< endl;
				exit(3);
			}

			cerr << "Restarting from where a previous run left off." << endl;
			string restartingPopFile = commandLineParser.GetSafeArgument("-X", 0, "0");
			cout << "Restarting with population from file named: " << restartingPopFile << endl;

			HCUBE::ExperimentRun experimentRun;
			if (commandLineParser.HasSwitch("-I")) {

				string inFile = commandLineParser.GetSafeArgument("-I", 0, "input.dat");
				cout << "overriding values from dat file: " << inFile << endl;
				experimentRun.setupExperimentInProgressUseDat(inFile, restartingPopFile, "restarted");
			} else {
				experimentRun.setupExperimentInProgress(restartingPopFile, "restarted");
			}

			if (commandLineParser.HasSwitch("-R")) {
				NEAT::Globals::getSingleton()->seedRandom(
						stringTo<unsigned int>(commandLineParser.GetSafeArgument("-R", 0, "0")));
			}

			SET_PARAMETER("MaxGenerations", extraGenerationsToRun + 1);

			if (commandLineParser.HasSwitch("-T")) {
				SET_PARAMETER("TerminalFitness", atof(commandLineParser.GetSafeArgument("-T", 0, "0").c_str()));
			}

			experimentRun.start();

		} else {
			printHelp();
		}
	} catch (string s) {
		cout << "CAUGHT ERROR AT " << __FILE__ << " : " << __LINE__ << endl;
		cout << "An exception has occured: " << s << endl;
	} catch (LocatedException e) {
		cout << "CAUGHT Exception: " << e.what() << endl;
	} catch (...) {
		cout << "CAUGHT ERROR AT " << __FILE__ << " : " << __LINE__ << endl;
		cout << "An error has occured!\n";
	}

	NEAT::Globals::deinit();

	return 0;
}

//IMPLEMENT_APP(MainApp)

int main(int argc, char **argv) {
#ifdef ONAMAC
	ProcessSerialNumber PSN;
	GetCurrentProcess(&PSN);
	TransformProcessType(&PSN,kProcessTransformToForegroundApplication);
#endif
	HyperNEAT_main(argc, argv);
}
